@@ -257,8 +257,8 @@ char *ngx_conf_check_num_bounds(ngx_conf_t *cf, void *post, void *data);
     }
 
 #define ngx_conf_merge_ptr_value(conf, prev, default)                        \
-    if (conf == NULL) {                                                      \
-        conf = (prev == NULL) ? default : prev;                              \
+    if (conf == NGX_CONF_UNSET_PTR) {                                        \
+        conf = (prev == NGX_CONF_UNSET_PTR) ? default : prev;                \
     }
 
 #define ngx_conf_merge_uint_value(conf, prev, default)                       \
