@@ -20,21 +20,30 @@
 typedef ngx_http_request_t   *nginx;
 
 typedef struct {
-    PerlInterpreter          *perl;
-
     ngx_str_t                 filename;
     ngx_str_t                 redirect_uri;
     ngx_str_t                 redirect_args;
+
     SV                       *next;
+    int                       sleep;
 
     ngx_uint_t                done;   /* unsigned  done:1; */
 
+    ngx_array_t              *variables;  /* array of ngx_http_perl_var_t */
+
 #if (NGX_HTTP_SSI)
     ngx_http_ssi_ctx_t       *ssi;
 #endif
 } ngx_http_perl_ctx_t;
 
 
+typedef struct {
+    ngx_uint_t    hash;
+    ngx_str_t     name;
+    ngx_str_t     value;
+} ngx_http_perl_var_t;
+
+
 extern ngx_module_t  ngx_http_perl_module;
 
 
@@ -48,7 +57,7 @@ extern ngx_module_t  ngx_http_perl_module;
 #endif
 
 
-extern void  boot_DynaLoader(pTHX_ CV* cv);
+extern void boot_DynaLoader(pTHX_ CV* cv);
 
 
 void ngx_http_perl_handle_request(ngx_http_request_t *r);
