@@ -275,9 +275,14 @@ typedef struct {
 } ngx_http_connection_t;
 
 
+typedef struct ngx_http_server_name_s  ngx_http_server_name_t;
+
+
 typedef struct {
-    ngx_hash_t                        hash;
-    ngx_hash_wildcard_t              *dns_wildcards;
+     ngx_hash_combined_t              names;
+
+     ngx_uint_t                       nregex;
+     ngx_http_server_name_t          *regex;
 } ngx_http_virtual_names_t;
 
 
